#include <avr/io.h>
#include <avr/interrupt.h>
#include <LiquidCrystal_I2C.h>
#include <dht11.h>
#include <Servo.h>
Servo motor;

dht11 DHT;
#define BUTTON_PIN 4  // Pin connected to the button
#define DHT11_PIN 7
float speed = 0.0;
LiquidCrystal_I2C lcd(0x27,16,2);
bool motorWorking = true;
float targetTemperature = 15.0;  // Desired temperature
float Kp = 5.0;  // Proportional gain
float Ki = 1.0;  // Integral gain
float Kd = 1.0;  // Derivative gain

float error, derivative;
float previousError = 0.0;
float integral = 0.0;
float position = 90.0;
void displaySpeedOnLcd();
volatile bool isMotorRunning = false;

volatile unsigned long interruptCounter = 0;
const unsigned long interruptInterval = 60000; // Intervalul de întreruperi (1 minute)
void controlMotor();


void setup() {
  lcd.init();
  lcd.backlight();
  lcd.setCursor(2,0);   //Set cursor to character 2 on line 0
  lcd.print("Temp: ");
  lcd.setCursor(2,1);   //Move cursor to character 2 on line 1
  lcd.print("Speed: ");
  motor.attach(3);
  pinMode(BUTTON_PIN, INPUT);
  int chk = DHT.read(DHT11_PIN);
  lcd.setCursor(8,0);
  lcd.print(DHT.temperature);
  lcd.print("C");

  cli(); // Dezactivează întreruperile globale

  // Setează modul de funcționare CTC (Clear Timer on Compare)
  TCCR0A = (1 << WGM01);

  // Setează valoarea de comparare a registrului OCR0A
  OCR0A = 31249; // Pentru o frecvență de întrerupere de 16 MHz/256/2 Hz - 1

  // Activează întreruperea la nivelul de comparare
  TIMSK0 = (1 << OCIE0A);

  // Setează prescalerul la 1024
  TCCR0B = ((1 << CS02) | (1 << CS00));

  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT20);

  sei(); // Activează întreruperile globale
}

void controlMotor() {
  float currentTemperature = DHT.temperature;
  error = targetTemperature - currentTemperature;
  derivative = error - previousError;
  integral += error;
  previousError = error;
  speed = abs(Kp * error + Ki * integral + Kd * derivative);
  speed = constrain(speed, 0, 121);
  position = map(speed, 0, 121, 100, 137);
  speed = map(speed, 0, 121, 0, 100);
  motor.write(position);
}

void displaySpeedOnLcd() {
  lcd.setCursor(9, 1);
  lcd.print("    ");
  lcd.setCursor(9, 1);
  lcd.print((int) speed);
  lcd.print("%");
}

void loop() {
  displaySpeedOnLcd();
}

// Rutina de întrerupere pentru Timer0
ISR(TIMER0_COMPA_vect) {
  interruptCounter++; // Incrementăm contorul de întreruperi
  
  if (interruptCounter >= interruptInterval) {
    interruptCounter = 0;
    if (DHT.temperature != targetTemperature && isMotorRunning) {
      controlMotor();
    }
  }
}

ISR(PCINT2_vect) {
  if (bit_is_clear(PIND, BUTTON_PIN) == 0) {
    isMotorRunning = !isMotorRunning; // Inversează starea motorului la fiecare apăsare a butonului
    if (isMotorRunning) {
      controlMotor();
    } else {
      speed = 0.0;
      motor.write(90);
    }
  }
}
